#include "../StdAfx.h"
#include "../Td3dDrawable.h"

#include "Tcircle.h"
#include "TgraphicsObject.h"

ImplementCompoundObjAttr(TgraphicsObject, TxformInfo,	XformInfo)
ImplementPointerObjAttr(TgraphicsObject, Tcircle,		CollisionCirclesInOC)
ImplementSimpleObjAttr(TgraphicsObject, 	  int,		NumCollisionCircles)
ImplementCompoundObjAttr(TgraphicsObject, D3DXVECTOR3,	Velocity)
ImplementBoolObjAttr(TgraphicsObject, Visibility)
ImplementBoolObjAttr(TgraphicsObject, ShowCollisionCircles)


TgraphicsObject::~TgraphicsObject()
{
	if (fCollisionCirclesInOC != NULL)
		delete [] fCollisionCirclesInOC;
}

TgraphicsObject::TgraphicsObject() :
	fVelocity(0, 0, 0),
	fCollisionCirclesInOC(NULL),
	fNumCollisionCircles(0),
	fVisibility(true),
	fShowCollisionCircles(false)
{
}


void TgraphicsObject::SetFirstPosition(const D3DXVECTOR3 &p) { fXformInfo.SetMoveTo(p);}
void TgraphicsObject::SetSecondPosition(const D3DXVECTOR3 &) {}
const D3DXVECTOR3& TgraphicsObject::GetFirstPosition() const { return fXformInfo.Movement(); }

bool TgraphicsObject::IntersectsBound(const T2dBound &b) { return b.PointInBound(GetXformInfo().Movement()); }
bool TgraphicsObject::ShouldDrawObject() const { return VisibilityIsEnabled(); }
void TgraphicsObject::SetObjectD3DAttributes(Td3dDrawable&, LPDIRECT3DDEVICE9) {} 
void TgraphicsObject::UnSetObjectD3DAttributes(Td3dDrawable&, LPDIRECT3DDEVICE9) {} 
void TgraphicsObject::DrawObjectGeom(Td3dDrawable&, LPDIRECT3DDEVICE9) {}
void TgraphicsObject::MoveObjectTo(const D3DXVECTOR3 &d) {fXformInfo.SetMoveTo(d); }

void TgraphicsObject::SetObjectD3DTransformation(ID3DXMatrixStack *stack) const {
					fXformInfo.SetUpD3DXformStack(stack);
}
	
void TgraphicsObject::SetDrawColor(const D3DXCOLOR& c) {}
D3DXCOLOR& TgraphicsObject::GetDrawColor() { static D3DXCOLOR c(0, 0, 0, 0); return c; }

void TgraphicsObject::SetPointSize(const float s) {}
float TgraphicsObject::GetPointSize() const { return 0.0f; }

void TgraphicsObject::SetFillMode(const WORD) {}
WORD TgraphicsObject::GetFillMode() const {return D3DFILL_SOLID;}
void TgraphicsObject::SetTextureIndex(const int) {}
int  TgraphicsObject::GetTextureIndex() { return -1; }
void TgraphicsObject::EnableAlphaBlending() {}
void TgraphicsObject::DisableAlphaBlending() {}
bool TgraphicsObject::AlphaBlendingIsEnabled() { return false; }
void TgraphicsObject::EnableTexture() {}
void TgraphicsObject::DisableTexture() {}
bool TgraphicsObject::TextureIsEnabled() { return false; }


void TgraphicsObject::SetShadedMode(const WORD) {}
WORD TgraphicsObject::GetShadedMode() const { return D3DSHADE_GOURAUD; }


//
// A little real work here ...
//
void
TgraphicsObject::UpdateLocation()
{
	fXformInfo.UpdateMovementBy(fVelocity);
}


bool
TgraphicsObject::CircleIntersectCircles(
	Tcircle		&c,
	int			num,
	Tcircle		*cs,
	D3DXVECTOR3 &delta,
	float 		&distance
)
{
	bool once = false;
	D3DXVECTOR3 dv;
	float		dist;
	bool 		touches;

	distance = M_VERY_LARGE;

	for (int i = 0; i<num; i++) {
		dv = c.GetCenter() - cs[i].GetCenter();
		dist = D3DXVec3Length(&dv);
		touches = (dist < (c.GetRadius()+cs[i].GetRadius()));
		if (touches && (dist < distance) ) {
			once = true;
			delta = dv;
			distance = dist;
		}					
	} 
	return once;
}

bool
TgraphicsObject::WCcirclesTouchObject(
	Td3dDrawable &drawable,
	Tcircle *c, 
	int n,
	D3DXVECTOR3 &delta, float &distance)
{
	int myNumCC;
	Tcircle myCC[20];	// going to assume no TgraphicsObject will have more than 20.
	GetCollisionCirclesInWC(drawable, myCC, myNumCC);
	
	bool once = false;
	//
	// if input we have no collision circles use them, otherwise return false
	//
	if (myNumCC > 0) {
		// we must intersect all of our collision circles with the incoming
		// collision circles and return the closes intersection.
		float dist = M_VERY_LARGE;
		D3DXVECTOR3 d;
		bool touches;
		distance = M_VERY_LARGE;
		for (int i=0; i<myNumCC; i++) {
			touches = CircleIntersectCircles(myCC[i], n, c, d, dist);
			if (touches && (dist < distance) ) {
				once = true;
				distance = dist;
				delta = d;
			}
		}
	}
	return once;
}

void TgraphicsObject::CreateCollisionCircles(int n)
{
	fNumCollisionCircles = n;
	fCollisionCirclesInOC = new Tcircle[n];
	for (int i=0; i<fNumCollisionCircles; i++) {
		fCollisionCirclesInOC[i].SetShadedMode(D3DSHADE_GOURAUD);
		fCollisionCirclesInOC[i].SetFillMode(D3DFILL_WIREFRAME);
		fCollisionCirclesInOC[i].SetDrawColor(D3DXCOLOR(0.8f, 0.3f, 0.3f, 0.0f));
	}
}

void TgraphicsObject::XformCollisionCirclesToWC(
	D3DXMATRIX *m,
	Tcircle    *c
)
{
	D3DXVECTOR3 ocC, ocP;
	D3DXVECTOR3 wcC, wcP;
	for (int i=0; i<GetNumCollisionCircles(); i++) {
		ocC = fCollisionCirclesInOC[i].GetCenter();
		ocP = ocC;
		ocP.x = ocC.x + fCollisionCirclesInOC[i].GetRadius();

		D3DXVec3TransformCoord(	&wcC, 			// output
								&ocC,			// input
								m);				// matrix
		D3DXVec3TransformCoord(	&wcP, 			// output
								&ocP,			// input
								m);				// matrix
		
		c[i] = fCollisionCirclesInOC[i];
		c[i].SetFirstPosition(wcC);
		c[i].SetSecondPosition(wcP);
	}
}

void TgraphicsObject::GetCollisionCirclesInWC(
	Td3dDrawable &drawable,
	Tcircle *cc,
	int &n
)
{
	ID3DXMatrixStack *toWorldStack = drawable.Obj2WldStack();
	// 1. compute our contribution to the transformation stack
	toWorldStack->Push();
		SetObjectD3DTransformation(toWorldStack);
		D3DXMATRIX *m = toWorldStack->GetTop();
		XformCollisionCirclesToWC(m, cc);
	toWorldStack->Pop();
	n = GetNumCollisionCircles();
}

void TgraphicsObject::DrawObject(
	Td3dDrawable &drawable
)
{
	if ( ShouldDrawObject() ) {

		LPDIRECT3DDEVICE9 d3dDevice = drawable.D3DDevice();
		ID3DXMatrixStack *toWorldStack = drawable.Obj2WldStack();

		toWorldStack->Push();
			// save a copy of the current xform

			SetObjectD3DTransformation(toWorldStack);

			d3dDevice->SetTransform(D3DTS_WORLD, toWorldStack->GetTop());

			SetObjectD3DAttributes(drawable, d3dDevice);

				DrawObjectGeom(drawable, d3dDevice);

			UnSetObjectD3DAttributes(drawable, d3dDevice);

		toWorldStack->Pop();
		d3dDevice->SetTransform(D3DTS_WORLD, toWorldStack->GetTop());
			// restore the previous xform info back into the matrix
		
		// Since collision circles are in world coordinate, we will draw them only after the stack is cleared
		if ( ShowCollisionCirclesIsEnabled() )
			DrawCollisionCircles(drawable);
			
	}
}

void TgraphicsObject::DrawCollisionCircles(
	Td3dDrawable &drawable
)
{
	int n;
	Tcircle c[20];

	GetCollisionCirclesInWC(drawable, c, n);

	ID3DXMatrixStack *toWorldStack = drawable.Obj2WldStack();
		// now since the circles are already in WC, we have to make
		// sure that this stack is cleared!!

	toWorldStack->Push();
	toWorldStack->LoadIdentity();

		for (int i=0; i<n; i++)
			c[i].DrawObject(drawable);

	toWorldStack->Pop();
}