#pragma once

/////////////////////////////////////////////////////////////////////////////////////////////
//
//  This class is copied and modified from
//  
//  	Special Effects Game Programming with DIRECT X
//  	By Mason McCuskey
//  	Published by: Premier Press
//  	Date: 2002
//  	ISBN: 1-931841-06-3
// 
// Source code from Chapter 11: Fire (Ch11p2_TextureFire.h)
//
// By: Kelvin Sung
// Date: August 2002
//
// Main modifications: 
//
//    . Let user decide where they want to add fuel, supporting adding
//       	effect source anywhere in the texture
// 
//    . Support alpha blending (in multi-stage texturing) allowing easier compositing
//       	to existing geometry/texture
//
//    . Define an additional "EffectSize" for "amount of Fuel"
//
//    . Removed dependence on the rest of the books' software
// 
//
//////////////////////////////////////////////////////////////////////


#include "GraphicsAPI.h"
#include "Ttexture.h"

class TupwardEffect : public Ttexture
{
	#define kEffectPaletteSize 256

	public:
  		TupwardEffect(LPDIRECT3DDEVICE9 d) : Ttexture(d) { 

			fFuelPos = NULL;
			fFuelSize = 0;

			fFuelDuration = 0;
   		 	fEffectBuffer1 = NULL; 
			fEffectBuffer2 = NULL;
    		fActiveBuffer = NULL; 
			fScratchBuffer = NULL; }

  		virtual ~TupwardEffect();

  		virtual bool CreateD3DTexture(int textureSize, int effectSize, int coolAmount);

  		virtual void UpdateTexture();

  		void SetCoolAmount(int coolamount) { fCoolAmount = coolamount; }
  		int  GetCoolAmount(void) const { return(fCoolAmount); }
  
  		void SetEffectSize(int size) { fEffectSize = size; }
  		int  GetEffectSize(void) const { return(fEffectSize); }

		void SetFuelDuration(int length) { fFuelDuration = length; }
  		void SetFuelSource(const D3DXVECTOR3 *fuelPos, int size);
				
	protected:
  
  		virtual void Process();
  		virtual bool CopyToTexture(void);

			//
			// color palette helps determine what effect we get
			//
		virtual void LoadEffectPalette() = 0;

			//
			// Fueling algorithm helps determine
		virtual int ComputeFuel(int index);

		void AddFuelToEffect();

		D3DXVECTOR3 *fFuelPos;		// array of fuel positions
		int			 fFuelSize;

		int 		 fFuelDuration;
		
  		int fCoolAmount;
  		int fTextureSize;
		int fEffectSize;

  		unsigned char *fEffectBuffer1; // first fire array
  		unsigned char *fEffectBuffer2; // 2nd fire array

  		unsigned char *fActiveBuffer;
  		unsigned char *fScratchBuffer;

  		PALETTEENTRY fEffectPalette[256];
};