#pragma once

#include "../GraphicsAPI/GraphicsAPI.h"
#include "../GraphicsAPI/TobjectList.h"
#include "../GraphicsAPI/Tcircle.h"
#include "../GraphicsAPI/Trectangle.h"
#include "../GraphicsAPI/TnormRectangle.h"

#include "../GraphicsAPI/TdrawTexture.h"
#include "../GraphicsAPI/TfireTexture.h"
#include "../GraphicsAPI/TsmokeTexture.h"


class Tworld {

	private:

		enum TdrawObjectType {
			eDrawCircle,
			eDrawRectangle,
			eDrawNormRectangle
		};

		TobjectList		fAllObject;

		TdrawObjectType fObjectType;

		TgraphicsObject *fCurrentObj;

		bool			fTextureEnabled;
		bool			fTransparencyEnabled;
		int				fTextureIndex;


		#define kDrawTextureSize	512
		TdrawTexture    fDrawTexture1, fDrawTexture2;

		#define kFireTextureSize 256
		TfireTexture	fFire1, fFire2;

		#define kSmokeTextureSize 256
		TsmokeTexture	fSmoke1, fSmoke2;
public:
		Tworld(Td3dDrawable&, Td3dDrawable&);

		
		void SetToDrawCircle() { fObjectType = eDrawCircle; }
		void SetToDrawRectangle() { fObjectType = eDrawRectangle; }
		void SetToDrawNormRectangle() { fObjectType = eDrawNormRectangle; }
		void SetScaleX( float s ) { fCurrentObj->GetXformInfo().SetScaleXTo(s); }
		void SetScaleY(  float s ) { fCurrentObj->GetXformInfo().SetScaleYTo(s); }
		void SetRotate( float s ) { fCurrentObj->GetXformInfo().SetRotationToDegree(s); }
		void SetTextureTo(bool b) { fTextureEnabled = b; 
									if (b)
										fCurrentObj->EnableTexture();
									else
										fCurrentObj->DisableTexture();
									}
		void SetTransparencyTo(bool b) { fTransparencyEnabled = b; 
									if (b)
										fCurrentObj->EnableAlphaBlending();
									else
										fCurrentObj->DisableAlphaBlending();
									}
		void SetTextureIndexTo(int i) { fTextureIndex = i; 
										fCurrentObj->SetTextureIndex(i);
									}

		void DrawOnTexture();
		void SetFireLocation();
		void SetSmokeLocation();

		//
		// Interactive drawing of object ..
		//
		void InitObject(float x, float y);
		void DrawObjectTo(float x, float y);
		void CompleteObjectAt(float x, float y);
		void MoveObjectTo(float x, float y);

		//
		// Redraws the entire world
		void DrawWorld(Td3dDrawable &d, LPDIRECT3DDEVICE9 d3dDevice);

		// Updates the entire world
		void UpdateWorld(Td3dDrawable &d, const T2dBound &b);
			// width and height is the dimension of the world (in pixels)
};