#include "StdAfx.h"
#include "Td3dDrawable.h"

void
Td3dDrawable::SetUpWorldToNDCTransform()
{
	//*****************************************************************
	// 1. Adjustments to satisfy the aspect ratio
	//
	// First thing we must figure out is aspect ratio
	// we want to use all the CWnd area. And we will
	// extend our World coordiante either in width or height
	// so that we can fill out the entire NDC CWnd area
	float cwndRatio = float(fHeight) / float(fWidth);
	float worldHeight = float(fWorldBound.Height());
	float worldWidth =  float (fWorldBound.Width());
	float worldRatio = worldHeight / worldWidth;

	if (cwndRatio > worldRatio) {
		// increase world height to meet the ratio
		worldHeight = worldWidth * cwndRatio;
	} else { 
		worldWidth = worldHeight / cwndRatio;
	}
	fWorldBound.SetBound(fWorldBound.Left(), fWorldBound.Left()+worldWidth, fWorldBound.Bottom(), fWorldBound.Bottom()+worldHeight);

	//*****************************************************************
	// 2. Now, we are ready to constract the matrix that will transform
	//    from the given world bound to the NDC
	D3DXMATRIX world2ndc, toSize, toOrg, toPlace;
	D3DXMatrixTranslation(&toOrg, -fWorldBound.Left(), -fWorldBound.Bottom(), 0.0);
			// 1. first operation is to line up the space's origins

	D3DXMatrixScaling(&toSize, 2.0f/worldWidth, 2.0/worldHeight, 1.0);
			// now scale to size 2

	D3DXMatrixTranslation(&toPlace, -1, -1, 0);
			// now move origin to -1 -1

	D3DXMatrixMultiply(&world2ndc, &toOrg, &toSize);
	D3DXMatrixMultiply(&world2ndc, &world2ndc, &toPlace);

	fD3DDevice->SetTransform(D3DTS_VIEW, &world2ndc);

	//
	// Here we do pre-computation for DeviceToWorldTransform
	//
	fDevice2WorldWidthScale = worldWidth / float(fWidth);
	fDevice2WorldHeightScale = worldHeight / float(fHeight);
}

void Td3dDrawable::DeviceToWorldTransform(
		const CPoint &devicePt, 
			  D3DXVECTOR3 &worldPt)
{
	// 1. we must scale Device point to wrold scale
	worldPt.x = devicePt.x * fDevice2WorldWidthScale;
	worldPt.y = devicePt.y * fDevice2WorldHeightScale;

	// 2. we add in the world lower-left corner offset
	worldPt.x += fWorldBound.Left();
	worldPt.y += fWorldBound.Bottom();
}